from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional
from xsdata.models.datatype import XmlDate, XmlTime

__NAMESPACE__ = "http://www.ismrm.org/ISMRMRD"


@dataclass
class accelerationFactorType:
    kspaceEncodingStep1: Optional[int] = field(
        default=None,
        metadata={
            "name": "kspace_encoding_step_1",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    kspaceEncodingStep2: Optional[int] = field(
        default=None,
        metadata={
            "name": "kspace_encoding_step_2",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


class calibrationModeType(Enum):
    EMBEDDED = "embedded"
    INTERLEAVED = "interleaved"
    SEPARATE = "separate"
    EXTERNAL = "external"
    OTHER = "other"


@dataclass
class coilLabelType:
    coilNumber: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    coilName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class experimentalConditionsType:
    h1ResonanceFrequencyHz: Optional[int] = field(
        default=None,
        metadata={
            "name": "H1resonanceFrequency_Hz",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class fieldOfViewMmType:
    class Meta:
        name = "fieldOfView_mmType"

    x: Optional[float] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    y: Optional[float] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    z: Optional[float] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


class interleavingDimensionType(Enum):
    PHASE = "phase"
    REPETITION = "repetition"
    CONTRAST = "contrast"
    AVERAGE = "average"
    OTHER = "other"


@dataclass
class limitType:
    minimum: int = field(
        default=0,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    maximum: int = field(
        default=0,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    center: int = field(
        default=0,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class matrixSizeType:
    x: int = field(
        default=1,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    y: int = field(
        default=1,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    z: int = field(
        default=1,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class measurementDependencyType:
    dependencyType: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    measurementId: Optional[str] = field(
        default=None,
        metadata={
            "name": "measurementID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


class patientPositionType(Enum):
    HFP = "HFP"
    HFS = "HFS"
    HFDR = "HFDR"
    HFDL = "HFDL"
    FFP = "FFP"
    FFS = "FFS"
    FFDR = "FFDR"
    FFDL = "FFDL"


@dataclass
class referencedImageSequenceType:
    referencedSopinstanceUid: List[str] = field(
        default_factory=list,
        metadata={
            "name": "referencedSOPInstanceUID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class sequenceParametersType:
    tr: List[float] = field(
        default_factory=list,
        metadata={
            "name": "TR",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    te: List[float] = field(
        default_factory=list,
        metadata={
            "name": "TE",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    ti: List[float] = field(
        default_factory=list,
        metadata={
            "name": "TI",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    flipAngleDeg: List[float] = field(
        default_factory=list,
        metadata={
            "name": "flipAngle_deg",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    sequenceType: Optional[str] = field(
        default=None,
        metadata={
            "name": "sequence_type",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    echoSpacing: List[float] = field(
        default_factory=list,
        metadata={
            "name": "echo_spacing",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class studyInformationType:
    studyDate: Optional[XmlDate] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    studyTime: Optional[XmlTime] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    studyId: Optional[str] = field(
        default=None,
        metadata={
            "name": "studyID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    accessionNumber: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    referringPhysicianName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    studyDescription: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    studyInstanceUid: Optional[str] = field(
        default=None,
        metadata={
            "name": "studyInstanceUID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class subjectInformationType:
    patientName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    patientWeightKg: Optional[float] = field(
        default=None,
        metadata={
            "name": "patientWeight_kg",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    patientId: Optional[str] = field(
        default=None,
        metadata={
            "name": "patientID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    patientBirthdate: Optional[XmlDate] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    patientGender: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "pattern": r"[MFO]",
        }
    )


class trajectoryType(Enum):
    CARTESIAN = "cartesian"
    EPI = "epi"
    RADIAL = "radial"
    GOLDENANGLE = "goldenangle"
    SPIRAL = "spiral"
    OTHER = "other"


@dataclass
class userParameterBase64Type:
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    value: Optional[bytes] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
            "format": "base64",
        }
    )


@dataclass
class userParameterDoubleType:
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    value: Optional[float] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class userParameterLongType:
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    value: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class userParameterStringType:
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


class waveformInformationTypeWaveformType(Enum):
    ECG = "ecg"
    PULSE = "pulse"
    RESPIRATORY = "respiratory"
    TRIGGER = "trigger"
    GRADIENTWAVEFORM = "gradientwaveform"
    OTHER = "other"


@dataclass
class acquisitionSystemInformationType:
    systemVendor: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    systemModel: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    systemFieldStrengthT: Optional[float] = field(
        default=None,
        metadata={
            "name": "systemFieldStrength_T",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    relativeReceiverNoiseBandwidth: Optional[float] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    receiverChannels: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    coilLabel: List[coilLabelType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    institutionName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    stationName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    deviceId: Optional[str] = field(
        default=None,
        metadata={
            "name": "deviceID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class encodingLimitsType:
    kspaceEncodingStep0: Optional[limitType] = field(
        default=None,
        metadata={
            "name": "kspace_encoding_step_0",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    kspaceEncodingStep1: Optional[limitType] = field(
        default=None,
        metadata={
            "name": "kspace_encoding_step_1",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    kspaceEncodingStep2: Optional[limitType] = field(
        default=None,
        metadata={
            "name": "kspace_encoding_step_2",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    average: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    slice: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    contrast: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    phase: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    repetition: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    set: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    segment: Optional[limitType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class encodingSpaceType:
    matrixSize: Optional[matrixSizeType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    fieldOfViewMm: Optional[fieldOfViewMmType] = field(
        default=None,
        metadata={
            "name": "fieldOfView_mm",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class measurementInformationType:
    measurementId: Optional[str] = field(
        default=None,
        metadata={
            "name": "measurementID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    seriesDate: Optional[XmlDate] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    seriesTime: Optional[XmlTime] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    patientPosition: Optional[patientPositionType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    initialSeriesNumber: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    protocolName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    seriesDescription: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    measurementDependency: List[measurementDependencyType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    seriesInstanceUidroot: Optional[str] = field(
        default=None,
        metadata={
            "name": "seriesInstanceUIDRoot",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    frameOfReferenceUid: Optional[str] = field(
        default=None,
        metadata={
            "name": "frameOfReferenceUID",
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    referencedImageSequence: Optional[referencedImageSequenceType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class parallelImagingType:
    accelerationFactor: Optional[accelerationFactorType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    calibrationMode: Optional[calibrationModeType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    interleavingDimension: Optional[interleavingDimensionType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class trajectoryDescriptionType:
    identifier: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    userParameterLong: List[userParameterLongType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    userParameterDouble: List[userParameterDoubleType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    comment: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class userParametersType:
    userParameterLong: List[userParameterLongType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    userParameterDouble: List[userParameterDoubleType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    userParameterString: List[userParameterStringType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    userParameterBase64: List[userParameterBase64Type] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class encodingType:
    encodedSpace: Optional[encodingSpaceType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    reconSpace: Optional[encodingSpaceType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    encodingLimits: Optional[encodingLimitsType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    trajectory: Optional[trajectoryType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    trajectoryDescription: Optional[trajectoryDescriptionType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    parallelImaging: Optional[parallelImagingType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )
    echoTrainLength: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
        }
    )


@dataclass
class waveformInformationType:
    waveformName: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    waveformType: Optional[waveformInformationTypeWaveformType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )
    userParameters: Optional[userParametersType] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://www.ismrm.org/ISMRMRD",
            "required": True,
        }
    )


@dataclass
class ismrmrdHeader:
    class Meta:
        namespace = "http://www.ismrm.org/ISMRMRD"

    version: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    subjectInformation: Optional[subjectInformationType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    studyInformation: Optional[studyInformationType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    measurementInformation: Optional[measurementInformationType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    acquisitionSystemInformation: Optional[acquisitionSystemInformationType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    experimentalConditions: Optional[experimentalConditionsType] = field(
        default=None,
        metadata={
            "type": "Element",
            "required": True,
        }
    )
    encoding: List[encodingType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "min_occurs": 1,
        }
    )
    sequenceParameters: Optional[sequenceParametersType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    userParameters: Optional[userParametersType] = field(
        default=None,
        metadata={
            "type": "Element",
        }
    )
    waveformInformation: List[waveformInformationType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "max_occurs": 32,
        }
    )
